/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of Mail Blaster.
 * 
 * Mail Blaster is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * Mail Blaster is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Mail Blaster. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.mailblaster.cli;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileFilter;

import com.gt.mailblaster.config.ConfigException;
import com.gt.mailblaster.config.SMTPConfig;

/**
 * The Configuration Builder UI.
 * 
 * @author The Shoe Shine Boy
 *
 */
public class ConfigBuilderFrame extends JFrame implements ActionListener, DocumentListener {
	private static final long serialVersionUID = 1L;

	private JTextField m_mailHostField = new JTextField("smtp.comcast.net");
	private JTextField m_acNameField = new JTextField("emblaster@comcast.net");
	private JPasswordField m_acPwdField = new JPasswordField("emblaster");
	private JCheckBox m_authCB = new JCheckBox("Authenticate", false);
	private JCheckBox m_enableTLSCB = new JCheckBox("Enable TLS", false);
	private JTextField m_subjectField = new JTextField("<no subject>");
	private JTextArea m_toArea = new JTextArea();
	private JTextArea m_atchArea = new JTextArea();
	private JCheckBox m_sendAsHTMLCB = new JCheckBox("Send Mail Body as HTML", false);
	private JEditorPane m_bodyPane = new JEditorPane();
	private JButton m_openBtn = new JButton("Open...");
	private JButton m_saveBtn = new JButton("Save");
	private JButton m_saveAsBtn = new JButton("Save As...");
	private JButton m_closeBtn = new JButton("Close");
	private String m_selectedFile = null;
	private JFileChooser m_fileChooser = new JFileChooser();
	private JPopupMenu m_bodyPopupMenu = new JPopupMenu();
	private JMenuItem m_bodyLoadFromFile = new JMenuItem("Load From File...");
	private JMenuItem m_bodyLoadFromURL = new JMenuItem("Load From URL...");
	private JPopupMenu m_atchPopupMenu = new JPopupMenu();
	private JMenuItem m_addAtch = new JMenuItem("Add Attachment...");
	
	private void saveConfig() {
		SMTPConfig config = new SMTPConfig();
		config.setMailHost(m_mailHostField.getText());
		config.setAcName(m_acNameField.getText());
		config.setAcPwd(new String(m_acPwdField.getPassword()));
		config.setIsAuthenticate(m_authCB.isSelected());
		config.setIsEnableTLS(m_enableTLSCB.isSelected());
		config.setSubject(m_subjectField.getText());
		StringTokenizer stTok = new StringTokenizer(m_toArea.getText().trim(), ",");
		ArrayList arr = new ArrayList();
		while (stTok.hasMoreTokens()) {
			arr.add(stTok.nextToken());
		}
		String[] tos = new String[arr.size()];
		for (int i=0; i<tos.length; i++) {
			tos[i] = (String)arr.get(i);
		}
		config.setTos(tos);
		StringTokenizer stTokAtchs = new StringTokenizer(m_atchArea.getText(), "\n");
		ArrayList atchFiles = new ArrayList();
		while (stTokAtchs.hasMoreTokens()) {
			String atchFileStr = stTokAtchs.nextToken();
			File f = new File(atchFileStr);
			if (f.exists() && !f.isDirectory()) {
				atchFiles.add(f);
			} else {
				// skip this with a warning
				System.err.println("Could not find " + atchFileStr + ". This will not " +
						"be included in the configuration file.");
			}
		}
		File[] atchFilesArr = new File[atchFiles.size()];
		for (int i=0; i<atchFiles.size(); i++) {
			atchFilesArr[i] = (File)atchFiles.get(i); 
		}
		config.setAttachments(atchFilesArr);
		config.setIsSendAsHTML(m_sendAsHTMLCB.isSelected());
		config.setBody(m_bodyPane.getText());
		try {
			config.saveTo(new File(m_selectedFile));
		} catch (ConfigException e) {
			JOptionPane.showMessageDialog(this, e.getMessage());
		}
		if (getTitle().endsWith("*")) {
			setTitle(getTitle().substring(0, getTitle().length()-2));
		} else {
			// do nothing
		}
	}
	
	private void updateTitleOnConfigChange() {
		if (!getTitle().endsWith("*")) {
			setTitle(getTitle() + " *");
		} else {
			// do nothing
		}
	}
	
	public ConfigBuilderFrame() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(new Dimension((int)(screenSize.width*0.75), (int)(screenSize.height*0.75)));
		setLocationRelativeTo(null);
		getContentPane().setLayout(new BorderLayout());
		
		JPanel northP = new JPanel();
		northP.setLayout(new BorderLayout());
		
		JPanel srvrSettingsP = new JPanel();
		
		JPanel mailHostP = new JPanel();
		mailHostP.add(new JLabel("Mail Host: "));
		mailHostP.add(m_mailHostField);
		srvrSettingsP.add(mailHostP);
		
		JPanel acNameP = new JPanel();
		acNameP.add(new JLabel("A/C Name: "));
		acNameP.add(m_acNameField);
		srvrSettingsP.add(acNameP);
		
		JPanel acPwdP = new JPanel();
		acPwdP.add(new JLabel("A/C Password: "));
		acPwdP.add(m_acPwdField);
		srvrSettingsP.add(acPwdP);
		
		srvrSettingsP.add(m_authCB);
		
		srvrSettingsP.add(m_enableTLSCB);
		
		northP.add(srvrSettingsP, BorderLayout.NORTH);
		
		JPanel toAreaP = new JPanel();
		toAreaP.setLayout(new BorderLayout());
		toAreaP.add(new JLabel("   Recipient(s) [comma separated]"), BorderLayout.NORTH);
		JScrollPane toAreaScrollPane = new JScrollPane();
		toAreaScrollPane.getViewport().add(m_toArea);
		m_toArea.setText("\n\n");
		m_toArea.setCaretPosition(0);
		toAreaP.add(toAreaScrollPane, BorderLayout.CENTER);
		toAreaP.add(new JPanel(), BorderLayout.WEST);
		toAreaP.add(new JPanel(), BorderLayout.EAST);
		
		northP.add(toAreaP, BorderLayout.CENTER);
		
		JPanel p = new JPanel();
		p.setLayout(new BorderLayout());
		p.add(new JPanel(), BorderLayout.NORTH);
		p.add(new JPanel(), BorderLayout.SOUTH);
		p.add(new JLabel("   Subject: "), BorderLayout.WEST);
		p.add(m_subjectField, BorderLayout.CENTER);
		p.add(m_sendAsHTMLCB, BorderLayout.EAST);
		
		northP.add(p, BorderLayout.SOUTH);
		
		getContentPane().add(northP, BorderLayout.NORTH);
		
		JPanel centerP = new JPanel();
		centerP.setLayout(new BorderLayout());
		
		JPanel attachmentsPanel = new JPanel();
		attachmentsPanel.setLayout(new BorderLayout());
		attachmentsPanel.add(new JLabel("Attachments (line separated):"), BorderLayout.NORTH);
		JScrollPane atchAreaScrollPane = new JScrollPane();
		atchAreaScrollPane.getViewport().add(m_atchArea);
		m_atchArea.setText("\n\n");
		m_atchArea.setCaretPosition(0);
		attachmentsPanel.add(atchAreaScrollPane, BorderLayout.CENTER);
		
		centerP.add(attachmentsPanel, BorderLayout.NORTH);
		
		JPanel pp = new JPanel();
		pp.setLayout(new BorderLayout());
		pp.add(new JLabel("Mail Body"), BorderLayout.NORTH);
		JScrollPane bodyScrollPane = new JScrollPane();
		bodyScrollPane.getViewport().add(m_bodyPane);
		pp.add(bodyScrollPane, BorderLayout.CENTER);
		
		centerP.add(pp, BorderLayout.CENTER);
		
		getContentPane().add(centerP, BorderLayout.CENTER);
		
		getContentPane().add(new JPanel(), BorderLayout.WEST);
		getContentPane().add(new JPanel(), BorderLayout.EAST);
		
		JPanel btnsPanel = new JPanel();
		m_openBtn.addActionListener(this);
		btnsPanel.add(m_openBtn);
		m_saveBtn.addActionListener(this);
		btnsPanel.add(m_saveBtn);
		m_saveAsBtn.addActionListener(this);
		btnsPanel.add(m_saveAsBtn);
		m_closeBtn.addActionListener(this);
		btnsPanel.add(m_closeBtn);
		getContentPane().add(btnsPanel, BorderLayout.SOUTH);
		
		m_fileChooser.setFileFilter(new FileFilter() {
			public boolean accept(File f) {
				return (f.isDirectory() || f.getName().toLowerCase().endsWith(".mb"));
			}
			public String getDescription() {
				return "Mail Blaster Config File";
			}
		});
		m_fileChooser.setAcceptAllFileFilterUsed(false);
		
		m_authCB.addActionListener(this);
		m_enableTLSCB.addActionListener(this);
		m_sendAsHTMLCB.addActionListener(this);
		
		setTitle("Mail Blaster");
		
		m_toArea.getDocument().addDocumentListener(this);
		m_bodyPane.getDocument().addDocumentListener(this);
		m_atchArea.getDocument().addDocumentListener(this);
		
		m_mailHostField.getDocument().addDocumentListener(this);
		m_acNameField.getDocument().addDocumentListener(this);
		m_acPwdField.getDocument().addDocumentListener(this);
		m_subjectField.getDocument().addDocumentListener(this);
		
		m_bodyLoadFromFile.addActionListener(this);
		m_bodyLoadFromURL.addActionListener(this);
		m_bodyPopupMenu.add(m_bodyLoadFromFile);
		m_bodyPopupMenu.add(m_bodyLoadFromURL);
		
		m_addAtch.addActionListener(this);
		m_atchPopupMenu.add(m_addAtch);
		
		m_atchArea.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (e.isMetaDown()) {
					m_atchPopupMenu.show(m_atchArea, e.getX(), e.getY());
				} else {
					// do nothing
				}
			}
		});
		
		m_bodyPane.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (e.isMetaDown()) {
					m_bodyPopupMenu.show(m_bodyPane, e.getX(), e.getY());
				} else {
					// do nothing
				}
			}
		});
	}

	public String getConfigFilePath() {
		setVisible(true);
		while (isVisible()) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			// do nothing
		}

		return m_selectedFile;
	}

	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource() instanceof JMenuItem) {
			if (arg0.getActionCommand().equals(m_addAtch.getText())) {
				JFileChooser fileC = new JFileChooser();
				int res = fileC.showOpenDialog(this);
				if (res == JFileChooser.APPROVE_OPTION) {
					try {
						String str = m_atchArea.getText().trim();
						if (str.length() == 0) {
							str = str + fileC.getSelectedFile().getCanonicalPath();
						} else {
							str = str + "\n" + fileC.getSelectedFile().getCanonicalPath();
						}
						m_atchArea.setText(str);
						updateTitleOnConfigChange();
					} catch (IOException e) {
						System.err.println(e.getMessage());
					}
				} else {
					// do nothing
				}
			} else if (arg0.getActionCommand().equals(m_bodyLoadFromFile.getText())) {
				JFileChooser fileC = new JFileChooser();
				int res = fileC.showOpenDialog(this);
				if (res == JFileChooser.APPROVE_OPTION) {
					try {
						m_bodyPane.setPage("file:///" +  fileC.getSelectedFile().getCanonicalPath());
					} catch (IOException e) {
						System.err.println(e.getMessage());
					}
					updateTitleOnConfigChange();
				} else {
					// do nothing
				}
			} else if (arg0.getActionCommand().equals(m_bodyLoadFromURL.getText())) {
				String url = JOptionPane.showInputDialog(this, "Enter URL", "Load From URL", JOptionPane.QUESTION_MESSAGE);
				if (url != null) {
					if (!url.startsWith("http://")) {
						url = "http://" + url;
					} else {
						// do nothing
					}
					try {
						m_bodyPane.setPage(url);
					} catch (IOException e1) {
						System.err.println(e1.getMessage());
					}
					updateTitleOnConfigChange();
				} else {
					// do nothing
				}
			} else {
				// do nothing
			}
		} else if (arg0.getSource() instanceof JButton) {
			if (arg0.getActionCommand().equals(m_openBtn.getText())) {
				if (m_fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
					SMTPConfig config = new SMTPConfig();
					try {
						config.loadFrom(m_fileChooser.getSelectedFile());
						m_mailHostField.setText(config.getMailHost());
						m_acNameField.setText(config.getAcName());
						m_acPwdField.setText(config.getAcPwd());
						m_authCB.setSelected(config.isAuthenticate());
						m_subjectField.setText(config.getSubject());
						m_enableTLSCB.setSelected(config.isEnableTLS());
						String str = "";
						for (int i=0; i<config.getTos().length; i++) {
							str = str + config.getTos()[i] + ",";
						}
						if (str.endsWith(", ")) {
							str = str.substring(0, str.length()-1);
						} else {
							// do nothing
						}
						m_toArea.setText(str);
						
						File[] atchFiles = config.getAttachments();
						str = "";
						for (int i=0; i<atchFiles.length; i++) {
							str = str + atchFiles[i].getCanonicalPath() + "\n";
						}
						m_atchArea.setText(str);
						
						m_sendAsHTMLCB.setSelected(config.isSendAsHTML());
						m_bodyPane.setText(config.getBody());
						
						setTitle("Mail Blaster - " + m_fileChooser.getSelectedFile().getCanonicalPath());
						
						m_selectedFile = m_fileChooser.getSelectedFile().getCanonicalPath();
					} catch (ConfigException e) {
						JOptionPane.showMessageDialog(this, e.getMessage());
					} catch (IOException e) {
						JOptionPane.showMessageDialog(this, e.getMessage());
					}
				} else {
					// do nothing
				}
			} else if (arg0.getActionCommand().equals(m_saveBtn.getText())) {
				if (m_selectedFile != null) {
					saveConfig();
				} else {
					if (m_fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
						try {
							m_selectedFile = m_fileChooser.getSelectedFile().getCanonicalPath();
							if (!m_selectedFile.endsWith(".mb")) {
								m_selectedFile = m_selectedFile + ".mb";
							} else {
								// do nothing
							}
							setTitle("Mail Blaster - " + m_selectedFile);
							saveConfig();
						} catch (IOException e) {
							JOptionPane.showMessageDialog(this, e.getMessage());
						}
					} else {
						// do nothing
					}
				}
			} else if (arg0.getActionCommand().equals(m_saveAsBtn.getText())) {
				if (m_fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
					try {
						m_selectedFile = m_fileChooser.getSelectedFile().getCanonicalPath();
						if (!m_selectedFile.endsWith(".mb")) {
							m_selectedFile = m_selectedFile + ".mb";
						} else {
							// do nothing
						}
						setTitle("Mail Blaster - " + m_selectedFile);
						saveConfig();
					} catch (IOException e) {
						JOptionPane.showMessageDialog(this, e.getMessage());
					}
				} else {
					// do nothing
				}
			} else if (arg0.getActionCommand().equals(m_closeBtn.getText())) {
				setVisible(false);
			} else {
				// do nothing
			}
		} else if (arg0.getSource() instanceof JCheckBox) {
			updateTitleOnConfigChange();
		} else {
			// do nothing
		}
	}

	public void changedUpdate(DocumentEvent e) {
		updateTitleOnConfigChange();
	}

	public void insertUpdate(DocumentEvent e) {
		updateTitleOnConfigChange();
	}

	public void removeUpdate(DocumentEvent e) {
		updateTitleOnConfigChange();
	}

	public void hide() {
		if (getTitle().endsWith("*")) {
			int res = JOptionPane.showConfirmDialog(this, "Do you want to save configuration changes (y/n)? "); 
			if (res == JOptionPane.YES_OPTION) {
				if (m_selectedFile == null) {
					if (m_fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
						try {
							m_selectedFile = m_fileChooser.getSelectedFile().getCanonicalPath();
							setTitle("Mail Blaster - " + m_selectedFile);
							saveConfig();
						} catch (IOException e) {
							JOptionPane.showMessageDialog(this, e.getMessage());
						}
					} else {
						return;
					}
				} else {
					saveConfig();
				}
			} else if (res == JOptionPane.CANCEL_OPTION) {
				return;
			} else {
				// do nothing
			}
		} else {
			// do nothing
		}
		super.hide();
	}

}
