/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of Mail Blaster.
 * 
 * Mail Blaster is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * Mail Blaster is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with Mail Blaster. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.mailblaster.config;

import java.io.File;

/**
 * The abstract class that defines configuration for mail blaster.
 * 
 * @author The Shoe Shine Boy
 *
 */
public abstract class AbstractConfig {

	/**
	 * Child classes must implement this method to load the configuration from a file.
	 * 
	 * @param file				The File containing configuration information.
	 * @throws ConfigException	Throw this exception if loading configuration fails.
	 */
	public abstract void loadFrom(File file) throws ConfigException;
	
	/**
	 * Child classes must implement this method to save the configuration into a file.
	 * 
	 * @param file				The File to which configuration must be saved.
	 * @throws ConfigException	Throw this exception if saving configuration fails.
	 */
	public abstract void saveTo(File file) throws ConfigException;

}
